@@ -1,11 +1,41 @@
 #!/usr/bin/env python
 # -*- Mode: Python; py-indent-offset: 4 -*-
-import sys, string
+import sys, os, string
 import getopt
 import defsparser, argtypes, override
 
 import traceback, keyword
 
+class FileOutput:
+    '''Simple wrapper for file object, that makes writing #line
+    statements easier.'''
+    def __init__(self, fp, filename=None):
+        self.fp = fp
+        self.lineno = 1
+        if filename:
+            self.filename = filename
+        else:
+            self.filename = self.fp.name
+    # handle writing to the file, and keep track of the line number ...
+    def write(self, str):
+        self.fp.write(str)
+        self.lineno = self.lineno + string.count(str, '\n')
+    def writelines(self, sequence):
+        for line in sequence:
+            self.write(line)
+    def close(self):
+        self.fp.close()
+    def flush(self):
+        self.fp.flush()
+
+    def setline(self, linenum, filename):
+        '''writes out a #line statement, for use by the C
+        preprocessor.'''
+        self.write('#line %d "%s"\n' % (linenum, filename))
+    def resetline(self):
+        '''resets line numbering to the original file'''
+        self.setline(self.lineno + 1, self.filename)
+
 # have to do return type processing
 functmpl = 'static PyObject *\n' + \
 	   '_wrap_%(cname)s(PyObject *self, PyObject *args, PyObject *kwargs)\n' + \
@@ -179,6 +209,8 @@ boxedconsttmpl = 'static int\n' + \
 boxedgetattrtmpl = 'static PyObject *\n' + \
                    '%(getattr)s(PyObject *self, char *attr)\n' + \
                    '{\n' + \
+                   '    if (!strcmp(attr, "__members__"))\n' + \
+                   '        return Py_BuildValue(%(members)s);\n' + \
                    '%(attrchecks)s' + \
                    '    PyErr_SetString(PyExc_AttributeError, attr);\n' + \
                    '    return NULL;\n' + \
@@ -370,8 +402,11 @@ def write_getsets(parser, objobj, castmacro, overrides, fp=sys.stdout):
         funcname = funcprefix + fname
         attrname = objobj.c_name + '.' + fname
         if overrides.attr_is_overriden(attrname):
+            lineno, filename = overrides.getstartline(attrname)
             code = overrides.attr_override(attrname)
+            fp.setline(lineno, filename)
             fp.write(code)
+            fp.resetline()
             getsets.append('    { "%s", (getter)%s, (setter)0 },\n' %
                            (fixname(fname), funcname))
             continue
@@ -409,7 +444,10 @@ def write_class(parser, objobj, overrides, fp=sys.stdout):
     if constructor:
 	try:
             if overrides.is_overriden(constructor.c_name):
+                lineno, filename = overrides.getstartline(constructor.c_name)
+                fp.setline(lineno, filename)
                 fp.write(overrides.override(constructor.c_name))
+                fp.resetline()
                 fp.write('\n\n')
             else:
                 write_constructor(objobj.c_name, castmacro, constructor, fp)
@@ -435,7 +473,10 @@ def write_class(parser, objobj, overrides, fp=sys.stdout):
 	try:
             methtype = 'METH_VARARGS'
             if overrides.is_overriden(meth.c_name):
+                lineno, filename = overrides.getstartline(meth.c_name)
+                fp.setline(lineno, filename)
                 fp.write(overrides.override(meth.c_name))
+                fp.resetline()
                 fp.write('\n\n')
                 if overrides.wants_kwargs(meth.c_name):
                     methtype = methtype + '|METH_KEYWORDS'
@@ -476,7 +517,10 @@ def write_interface(parser, interface, overrides, fp=sys.stdout):
 	try:
             methtype = 'METH_VARARGS'
             if overrides.is_overriden(meth.c_name):
+                lineno, filename = overrides.getstartline(meth.c_name)
+                fp.setline(lineno, filename)
                 fp.write(overrides.override(meth.c_name))
+                fp.resetline()
                 fp.write('\n\n')
                 if overrides.wants_kwargs(meth.c_name):
                     methtype = methtype + '|METH_KEYWORDS'
@@ -596,6 +640,7 @@ def write_boxed_getattr(parser, boxedobj, overrides, fp=sys.stdout):
     if not boxedobj.fields:
         return '0'
     attrchecks = ''
+    attrs = []
     for ftype, fname in boxedobj.fields:
         attrname = boxedobj.c_name + '.' + fname
         if overrides.attr_is_overriden(attrname):
@@ -604,6 +649,7 @@ def write_boxed_getattr(parser, boxedobj, overrides, fp=sys.stdout):
             attrchecks = attrchecks + attrchecktmpl % { 'attr': fixname(fname),
                                                         'varlist': '',
                                                         'code': code }
+            attrs.append(fname)
             continue
         try:
             varlist = argtypes.VarList()
@@ -616,12 +662,17 @@ def write_boxed_getattr(parser, boxedobj, overrides, fp=sys.stdout):
             attrchecks = attrchecks + attrchecktmpl % { 'attr': fixname(fname),
                                                         'varlist': varlist,
                                                         'code': code }
+            attrs.append(fname)            
         except:
             sys.stderr.write("couldn't write check for " + boxedobj.c_name +
                              '.' + fname + '\n')
 	    #traceback.print_exc()
     funcname = '_wrap_' + string.lower(uline) + '_getattr'
+    members = '"[%s]"' % ('s' * len(attrs))
+    for fname in attrs:
+        members = '%s, "%s"' % (members, fname)
     fp.write(boxedgetattrtmpl % {'getattr':    funcname,
+                                 'members':    members,
                                  'attrchecks': attrchecks })
     return funcname
 
@@ -633,7 +684,10 @@ def write_boxed(parser, boxedobj, overrides, fp=sys.stdout):
     if constructor:
 	try:
             if overrides.is_overriden(constructor.c_name):
+                lineno, filename = overrides.getstartline(constructor.c_name)
+                fp.setline(lineno, filename)
                 fp.write(overrides.override(constructor.c_name))
+                fp.resetline()
                 fp.write('\n\n')
             else:
                 write_boxed_constructor(boxedobj.c_name, boxedobj.typecode,
@@ -649,7 +703,10 @@ def write_boxed(parser, boxedobj, overrides, fp=sys.stdout):
 	try:
             methtype = 'METH_VARARGS'
             if overrides.is_overriden(meth.c_name):
+                lineno, filename = overrides.getstartline(meth.c_name)
+                fp.setline(lineno, filename)
                 fp.write(overrides.override(meth.c_name))
+                fp.resetline()
                 fp.write('\n\n')
                 if overrides.wants_kwargs(meth.c_name):
                     methtype = methtype + '|METH_KEYWORDS'
@@ -686,7 +743,10 @@ def write_functions(parser, overrides, prefix, fp=sys.stdout):
 	try:
             methtype = 'METH_VARARGS'
             if overrides.is_overriden(func.c_name):
+                lineno, filename = overrides.getstartline(func.c_name)
+                fp.setline(lineno, filename)
                 fp.write(overrides.override(func.c_name))
+                fp.resetline()
                 fp.write('\n\n')
                 if overrides.wants_kwargs(func.c_name):
                     methtype = methtype + '|METH_KEYWORDS'
@@ -719,10 +779,16 @@ def write_enums(parser, prefix, fp=sys.stdout):
                      % (enum.typecode,))
     fp.write('}\n\n')
 
-def write_source(parser, overrides, prefix, fp=sys.stdout):
+def write_source(parser, overrides, prefix, fp=FileOutput(sys.stdout)):
     fp.write('/* -*- Mode: C; c-basic-offset: 4 -*- */\n\n')
     fp.write('#include <Python.h>\n\n\n')
     fp.write(overrides.get_headers())
+    fp.resetline()
+    fp.write('\n\n')
+    fp.write('/* ---------- types from other modules ---------- */\n')
+    for module, pyname, cname in overrides.get_imports():
+        fp.write('static PyTypeObject *_%s;\n' % cname)
+        fp.write('#define %s (*_%s)\n' % (cname, cname))
     fp.write('\n\n')
     fp.write('/* ---------- forward type declarations ---------- */\n')
     for obj in parser.boxes:
@@ -748,7 +814,24 @@ def write_source(parser, overrides, prefix, fp=sys.stdout):
 
     fp.write('/* intialise stuff extension classes */\n')
     fp.write('void\n' + prefix + '_register_classes(PyObject *d)\n{\n')
+    imports = overrides.get_imports()[:]
+    if imports:
+        bymod = {}
+        for module, pyname, cname in imports:
+            bymod.setdefault(module, []).append((pyname, cname))
+        fp.write('    PyObject *module;\n\n')
+        for module in bymod:
+            fp.write('    if ((module = PyImport_ImportModule("%s")) != NULL) {\n' % module)
+            fp.write('        PyObject *moddict = PyModule_GetDict(module);\n\n')
+            for pyname, cname in bymod[module]:
+                fp.write('        _%s = (PyTypeObject *)PyDict_GetItemString(moddict, "%s");\n' % (cname, pyname))
+            fp.write('    } else {\n')
+            fp.write('        Py_FatalError("could not import %s");\n' %module)
+            fp.write('        return;\n')
+            fp.write('    }\n')
+        fp.write('\n')
     fp.write(overrides.get_init() + '\n')
+    fp.resetline()
 
     for boxed in parser.boxes:
         fp.write('    pyg_register_boxed(d, "' + boxed.name +
@@ -798,14 +881,15 @@ def register_types(parser):
 	else:
 	    argtypes.matcher.register_enum(enum.c_name, enum.typecode)
 
-if __name__ == '__main__':
-    o = override.Overrides(None)
+def main():
+    o = override.Overrides()
     prefix = 'pygtk'
+    outfilename = None
     opts, args = getopt.getopt(sys.argv[1:], "o:p:r:",
-                               ["override=", "prefix=", "register="])
+                        ["override=", "prefix=", "register=", "outfilename="])
     for opt, arg in opts:
         if opt in ('-o', '--override'):
-            o = override.Overrides(open(arg))
+            o = override.Overrides(arg)
         elif opt in ('-p', '--prefix'):
             prefix = arg
         elif opt in ('-r', '--register'):
@@ -813,11 +897,18 @@ if __name__ == '__main__':
             p.startParsing()
             register_types(p)
             del p
+        elif opt == '--outfilename':
+            outfilename = arg
     if len(args) < 1:
         sys.stderr.write(
             'usage: codegen.py [-o overridesfile] [-p prefix] defsfile\n')
         sys.exit(1)
     p = defsparser.DefsParser(args[0])
+    if not outfilename:
+        outfilename = os.path.splitext(args[0])[0] + '.c'
     p.startParsing()
     register_types(p)
-    write_source(p, o, prefix)
+    write_source(p, o, prefix, FileOutput(sys.stdout, outfilename))
+
+if __name__ == '__main__':
+    main()
