@@ -308,8 +308,8 @@ pygobject_new(GObject *obj)
 	    self->hasref = FALSE;
 	    g_object_steal_qdata(obj, pygobject_ownedref_key);
 	    g_object_ref(obj);
-	}
-	Py_INCREF(self);
+	} else
+	    Py_INCREF(self);
 	return (PyObject *)self;
     }
 
@@ -318,11 +318,15 @@ pygobject_new(GObject *obj)
 
     if (self == NULL)
 	return NULL;
-    self->obj = obj;
-    g_object_ref(obj);
+    self->obj = g_object_ref(obj);
+    self->hasref = FALSE;
+    self->inst_dict = NULL;
+    self->weakreflist = NULL;
     /* save wrapper pointer so we can access it later */
     g_object_set_qdata(obj, pygobject_wrapper_key, self);
 
+    PyObject_GC_Track((PyObject *)self);
+
     return (PyObject *)self;
 }
 
@@ -963,7 +967,13 @@ pyg_closure_new(PyObject *callback, PyObject *extra_args, PyObject *swap_data)
     Py_INCREF(callback);
     ((PyGClosure *)closure)->callback = callback;
     if (extra_args) {
-	Py_INCREF(extra_args);
+	if (!PyTuple_Check(extra_args)) {
+	    PyObject *tmp = PyTuple_New(1);
+	    PySequence_SetItem(tmp, 0, extra_args);
+	    extra_args = tmp;
+	} else {
+            Py_INCREF(extra_args);
+	}
 	((PyGClosure *)closure)->extra_args = extra_args;
     }
     if (swap_data) {
@@ -1105,7 +1115,9 @@ pygobject_dealloc(PyGObject *self)
 	Py_DECREF(self->inst_dict);
     self->inst_dict = NULL;
 
-    self->ob_type->tp_free((PyObject *)self);
+    /* the following causes problems with subclassed types */
+    /*self->ob_type->tp_free((PyObject *)self); */
+    PyObject_GC_Del(self);
 }
 
 static int
@@ -1125,10 +1137,13 @@ pygobject_hash(PyGObject *self)
 static PyObject *
 pygobject_repr(PyGObject *self)
 {
-    gchar buf[128];
+    gchar buf[256];
 
-    g_snprintf(buf, sizeof(buf), "<%s at 0x%lx>", G_OBJECT_TYPE_NAME(self->obj),
-	       (long)self->obj);
+    g_snprintf(buf, sizeof(buf),
+	       "<%s object (%s) at 0x%lx>",
+	       self->ob_type->tp_name,
+	       self->obj ? G_OBJECT_TYPE_NAME(self->obj) : "uninitialized",
+	       (long)self);
     return PyString_FromString(buf);
 }
 
@@ -1519,7 +1534,8 @@ pygobject_emit(PyGObject *self, PyObject *args)
     g_value_set_object(&params[0], G_OBJECT(self->obj));
 
     for (i = 0; i < query.n_params; i++)
-	g_value_init(&params[i + 1], query.param_types[i]);
+	g_value_init(&params[i + 1],
+		     query.param_types[i] & ~G_SIGNAL_TYPE_STATIC_SCOPE);
     for (i = 0; i < query.n_params; i++) {
 	PyObject *item = PyTuple_GetItem(args, i+1);
 
@@ -1538,12 +1554,12 @@ pygobject_emit(PyGObject *self, PyObject *args)
 	}
     }
     if (query.return_type != G_TYPE_NONE)
-	g_value_init(&ret, query.return_type);
+	g_value_init(&ret, query.return_type & ~G_SIGNAL_TYPE_STATIC_SCOPE);
     g_signal_emitv(params, signal_id, detail, &ret);
     for (i = 0; i < query.n_params + 1; i++)
 	g_value_unset(&params[i]);
     g_free(params);
-    if (query.return_type != G_TYPE_NONE) {
+    if (query.return_type & ~G_SIGNAL_TYPE_STATIC_SCOPE != G_TYPE_NONE) {
 	py_ret = pyg_value_as_pyobject(&ret);
 	g_value_unset(&ret);
     } else {
@@ -2086,6 +2102,77 @@ pyg_object_class_list_properties (PyObject *self, PyObject *args)
     return list;
 }
 
+static PyObject *
+pyg_object_new (PyGObject *self, PyObject *args, PyObject *kwargs)
+{
+    PyObject *pytype;
+    GType type;
+    GObject *obj = NULL;
+    GObjectClass *class;
+    PyObject *value;
+    PyObject *key;
+    int pos=0, num_params=0, i;
+    GParameter *params;
+
+    if (!PyArg_ParseTuple (args, "O:gobject.new", &pytype)) {
+	return NULL;
+    }
+
+    if ((type = pyg_type_from_object (pytype)) == 0)
+	return NULL;
+    
+    if ((class = g_type_class_ref (type)) == NULL) {
+	PyErr_SetString(PyExc_TypeError,
+			"could not get a reference to type class");
+	return NULL;
+    }
+
+    params = g_new0(GParameter, PyDict_Size(kwargs));
+    
+    while (kwargs && PyDict_Next (kwargs, &pos, &key, &value)) {
+	GParamSpec *pspec;
+	gchar *key_str = g_strdup(PyString_AsString (key));
+	pspec = g_object_class_find_property (class, key_str);
+	if (!pspec) {
+	    gchar buf[512];
+
+	    g_snprintf(buf, sizeof(buf),
+		       "gobject `%s' doesn't support property `%s'",
+		       g_type_name(type), key_str);
+	    PyErr_SetString(PyExc_TypeError, buf);
+	    goto cleanup;
+	}
+	g_value_init(&params[num_params].value,
+		     G_PARAM_SPEC_VALUE_TYPE(pspec));
+	if (pyg_value_from_pyobject(&params[num_params].value, value)) {
+	    gchar buf[512];
+
+	    g_snprintf(buf, sizeof(buf),
+		       "could not convert value for property `%s'", key_str);
+	    PyErr_SetString(PyExc_TypeError, buf);
+	    goto cleanup;
+	}
+	params[num_params].name = key_str;
+	num_params++;
+    }
+
+    obj = g_object_newv(type, num_params, params);
+    if (!obj)
+	PyErr_SetString (PyExc_RuntimeError, "could not create object");
+	   
+ cleanup:
+    for (i = 0; i < num_params; i++) {
+	g_free((gchar *) params[i].name);
+	g_value_unset(&params[i].value);
+    }
+    g_free(params);
+    g_type_class_unref(class);
+    
+    if (obj)
+	return pygobject_new ((GObject *)obj);
+    return NULL;
+}
+
 static PyMethodDef pygobject_functions[] = {
     { "type_name", pyg_type_name, METH_VARARGS },
     { "type_from_name", pyg_type_from_name, METH_VARARGS },
@@ -2097,6 +2184,7 @@ static PyMethodDef pygobject_functions[] = {
     { "signal_new", pyg_signal_new, METH_VARARGS },
     { "signal_list_names", pyg_signal_list_names, METH_VARARGS },
     { "list_properties", pyg_object_class_list_properties, METH_VARARGS },
+    { "new", (PyCFunction)pyg_object_new, METH_VARARGS|METH_KEYWORDS },
     { NULL, NULL, 0 }
 };
 
